--[[ deaths counter.lua
Author: birbhorse, aka EeveeEuphoria
Modified for use in MMT Revamped.

replaces the lives counter with a death counter, while also giving the player infinite lives
does not apply to game modes where lives aren't applicable

]]--


-- [[ player logic ]] --

addHook("PlayerThink", function(player)	
	if not G_GametypeUsesLives() then hud.enable("lives") return end
	if not player.DCMod return end
	if not player.DCMod.initialize return end
	if player.playerstate == PST_DEAD and not player.DCMod.timer and not G_IsSpecialStage(gamemap) and not player.DCMod.reverse
		player.DCMod.deaths = $ + 1
	end
	
	if player.playerstate == PST_DEAD and not player.DCMod.reverse
		player.DCMod.timer = $ + 1
		player.DCMod.Died = true
	end

	
	if player.DCMod.reverse
		if player.playerstate != PST_DEAD
			player.DCMod.timer2 = $ + 1
			if player.DCMod.timer2 > TICRATE*2
				if player.DCMod.timer < TICRATE
					player.DCMod.timer = $ + 1
				else
					player.DCMod.reverse = false
					player.DCMod.timer = 0
				end
			end
		else
			player.DCMod.reverse = false
			player.DCMod.timer = 0
		end
	end
	
	if (player.pflags & PF_FINISHED) or stoppedclock
		player.DCMod.timer = $ + 1
	end
end)


addHook("PlayerThink", function(player) -- replaced player spawn with a thinker, just so it can run after mmt
	if not G_GametypeUsesLives() then hud.enable("lives") return end

	if player.MMTloaded then
		if player.lives != INFLIVES then
			player.DCMod.PreviousLives = player.lives
		end

		if player.deathscounter != "off" then
			hud.disable("lives")
			player.lives = INFLIVES
		else
			if player.DCMod.liveshudenabled then
				hud.enable("lives")
			end
			if tonumber(player.DCMod.PreviousLives) and player.lives == INFLIVES then
				player.lives = player.DCMod.PreviousLives
			end
		end
	end
end)

addHook("PlayerSpawn", function(player)
	if not player.DCMod then
		player.DCMod = {}
		player.DCMod.deaths = 0
		player.DCMod.Died = true
		player.DCMod.CurrentLevel = gamemap
		player.DCMod.liveshudenabled = hud.enabled("lives")
	end
	
	if player.DCMod.CurrentLevel != gamemap then
		player.DCMod.CurrentLevel = gamemap 
		if player.deathscounter == "reset after level" then
			player.DCMod.deaths = 0
		end
		player.DCMod.Died = true
	end
	
	if not player.DCMod.Died and not G_IsSpecialStage(gamemap) then -- detects if a player hit the "retry" button in single player
		player.DCMod.deaths = $ + 1
	else
		player.DCMod.Died = false
	end

	player.DCMod.timer = 0
	player.DCMod.timer2 = 0
	player.DCMod.reverse = true
	
	
	player.DCMod.initialize = true
end)

-- [[ hud logic ]] --

hud.add(function(v, player, cam) if not G_GametypeUsesLives() hud.enable("lives") return end if not player.DCMod then return end if player.DCMod.timer == nil then return end
	if player.bot return end
	if not (player.mo and player.mo.valid and player and player.valid) return end -- just for in case, srb2 can be weird about this 
	if player.deathscounter == "off" then return end
	
	local XGoal = hudinfo[HUD_LIVES].x
	local YGoal = hudinfo[HUD_LIVES].y
	
	local function CheckPause() -- detect if game is paused, if so, halt all operations, unless the game is alt-tabbed for some fucking reason
		if menuactive and not netgame then return true end
		if paused then return true end
		--if focuslost return true end
		return false
	end
	
	v.TickDownValue = 20 -- after this amount of tics, assuming it's less than TICRATE, the sine formula will start to go in reverse, aka negative values
	v.TimerPlus = 65 -- additional value added to sinmath value, to increase the amount of time it's actually been since things started for that value alone
	
	if v.xoffset == nil
		v.xoffset = 0
		v.yoffset = YGoal*FRACUNIT
	end
	
	if player.MMTdcvis != "show" then
		if not CheckPause()
			if player.DCMod.timer == 0
				if player.DCMod.reverse
					v.yoffset = YGoal*FRACUNIT
				else
					v.yoffset = 200*FRACUNIT -- off-screen, hopefully
				end
			else
				if player.DCMod.reverse
					if player.DCMod.timer <= TICRATE
						v.tickerdown = 15 - player.DCMod.timer
						v.SinMath = (FRACUNIT/3)*(player.DCMod.timer+(v.TimerPlus*2))
						v.yoffset = $ - (v.tickerdown * 100)*sin(v.SinMath)
					end
				else
					if player.DCMod.timer <= TICRATE
						v.tickerdown = v.TickDownValue - player.DCMod.timer
						v.SinMath = (FRACUNIT)*(player.DCMod.timer+v.TimerPlus)
						v.yoffset = $ - (v.tickerdown * 85)*sin(v.SinMath)
						if player.DCMod.timer > v.TickDownValue then
							if v.yoffset > YGoal*FRACUNIT
								v.yoffset = YGoal*FRACUNIT
							end
						end
					end
				end
			end
		end
	else
		v.yoffset = YGoal*FRACUNIT
	end
	local booblean = false
	if player.powers[pw_super] then
		booblean = true
	end

	local lifeicon = v.getSprite2Patch(player.mo.skin, "XTRA", booblean, A)
	local colormap = v.getColormap(player.mo.skin, player.mo.color)
	local xicon = v.cachePatch("stlivex")
	
	v.drawScaled((XGoal+v.xoffset)*FRACUNIT, v.yoffset, FRACUNIT/2, lifeicon, V_SNAPTOLEFT|V_SNAPTOBOTTOM, colormap)  -- life icon
	v.drawScaled((XGoal+22+v.xoffset)*FRACUNIT, v.yoffset + (9*FRACUNIT), FRACUNIT, xicon, V_SNAPTOLEFT|V_SNAPTOBOTTOM)  -- "X"
	v.drawString((XGoal+18+v.xoffset), (v.yoffset/FRACUNIT) + 0, "DEATHS", V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_YELLOWMAP)  -- text
	v.drawString((XGoal+42+v.xoffset), (v.yoffset/FRACUNIT) + 8, player.DCMod.deaths, V_SNAPTOLEFT|V_SNAPTOBOTTOM) -- number

end, "game")

local function AlwaysOnDeathsCounter(v, player, cam) if not player then player = displayplayer end
	if not G_GametypeUsesLives() then hud.enable("lives") return end
	if not player.DCMod then return end if player.DCMod.timer == nil then return end if player.bot then return end
	if not (player.mo and player.mo.valid and player and player.valid) then return end -- just for in case, srb2 can be weird about this 
	if player.deathscounter == "off" then return end
	if player.MMTdcvis == "on" then return end 
	
	if player.DCMod.timer2 < TICRATE*3 then return end
	
	
	local XGoal = hudinfo[HUD_LIVES].x
	local YGoal = hudinfo[HUD_LIVES].y
	
	if v.xxoffset == nil
		v.xxoffset = 0
		v.yyoffset = YGoal*FRACUNIT
	end

	v.yyoffset = YGoal*FRACUNIT

	local lifeicon = v.getSprite2Patch(player.mo.skin, "XTRA", A)
	local colormap = v.getColormap(player.mo.skin, player.mo.color)
	local xicon = v.cachePatch("stlivex")
	
	v.drawScaled((XGoal+v.xxoffset)*FRACUNIT, v.yyoffset, FRACUNIT/2, lifeicon, V_SNAPTOLEFT|V_SNAPTOBOTTOM, colormap)  -- life icon
	v.drawScaled((XGoal+22+v.xxoffset)*FRACUNIT, v.yyoffset + (9*FRACUNIT), FRACUNIT, xicon, V_SNAPTOLEFT|V_SNAPTOBOTTOM)  -- "X"
	v.drawString((XGoal+18+v.xxoffset), (v.yyoffset/FRACUNIT) + 0, "DEATHS", V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_YELLOWMAP)  -- text
	v.drawString((XGoal+42+v.xxoffset), (v.yyoffset/FRACUNIT) + 8, player.DCMod.deaths, V_SNAPTOLEFT|V_SNAPTOBOTTOM) -- number
end



hud.add(AlwaysOnDeathsCounter, "scores")
--hud.add(AlwaysOnDeathsCounter, "intermission") -- doesn't actually appear?