-- &
local thosbutons = {
	"up","down","left","right",
	"spin","jump",
	"attack","nextwep","prevwep",
	"custom1","custom2","custom3",
	"moving","hmoving","vmoving",
	"attacknormal", "camleft", "camright", "tossflag" -- birbhorse additions
}
for k,v in ipairs(thosbutons) do
	thosbutons[v] = true
end

--NOTE: this has been modified specifically for mmt, all "p.bt..." variables have been converted to "p.mmtbt..." so it doesn't conflict with any other mod that uses this system

rawset(_G, "pbtDaemonmmt", function(p)
	--if deepcopy == nil then error("deepcopy missing", 2) end
	-- Does a cool thing
	local cmdbt = p.cmd.buttons
	local cmdfm = p.cmd.forwardmove
	local cmdsm = p.cmd.sidemove
	
	if not p.mmtbttimestamp then p.mmtbttimestamp = -1 end
	
	if p.mmtbttimestamp ~= leveltime then
		-- Prevent this from running more than once if multiple daemons are in place (why would there be more than one)
		
		if not p.mmtbt then
			p.mmtbt = {}
			p.mmtbtx = {}
			p.mmtbtprev = {}
			for _,v in ipairs(thosbutons) do
				p.mmtbt[v] = 0
				p.mmtbtprev[v] = 0
				p.mmtbtx[v] = 0
			end
		end
		local sens = 20
		
		for i = 1,#thosbutons do
			local ix = thosbutons[i]
			p.mmtbtprev[ix] = p.mmtbt[ix]
		end -- Old values
		
		p.mmtbt.up = cmdfm > sens and $+1 or 0
		p.mmtbt.down = cmdfm < -sens and $+1 or 0
		p.mmtbt.right = cmdsm > sens and $+1 or 0
		p.mmtbt.left = cmdsm < -sens and $+1 or 0
		
		if p.mmtbt.up > 0 and p.mmtbt.down > 0 then p.mmtbt.up = 0; p.mmtbt.down = 0 end
		if p.mmtbt.right > 0 and p.mmtbt.left > 0 then p.mmtbt.right = 0; p.mmtbt.left = 0 end
		
		p.mmtbt.spin = (cmdbt & BT_USE) > 0 and $+1 or 0
		p.mmtbt.jump = (cmdbt & BT_JUMP) > 0 and $+1 or 0
		p.mmtbt.attack = (cmdbt & BT_ATTACK) > 0 and $+1 or 0
		p.mmtbt.attacknormal = (cmdbt & BT_FIRENORMAL) > 0 and $+1 or 0
		
		p.mmtbt.tossflag = (cmdbt & BT_TOSSFLAG) > 0 and $+1 or 0
		
		p.mmtbt.camleft = (cmdbt & BT_CAMLEFT) > 0 and $+1 or 0
		p.mmtbt.camright = (cmdbt & BT_CAMRIGHT) > 0 and $+1 or 0
		
		p.mmtbt.nextwep = (cmdbt & BT_WEAPONNEXT) > 0 and $+1 or 0
		p.mmtbt.prevwep = (cmdbt & BT_WEAPONPREV) > 0 and $+1 or 0
		
		p.mmtbt.custom1 = (cmdbt & BT_CUSTOM1) > 0 and $+1 or 0
		p.mmtbt.custom2 = (cmdbt & BT_CUSTOM2) > 0 and $+1 or 0
		p.mmtbt.custom3 = (cmdbt & BT_CUSTOM3) > 0 and $+1 or 0
		
		p.mmtbt.hmoving = (p.mmtbt.left or p.mmtbt.right) and $~=nil and $+1 or 0
		p.mmtbt.vmoving = (p.mmtbt.up or p.mmtbt.down) and $~=nil and $+1 or 0
		p.mmtbt.moving = (p.mmtbt.up or p.mmtbt.down or p.mmtbt.left or p.mmtbt.right) and $~=nil and $+1 or 0
		
		local sum = 0
		local sidesum = 0
		
		if cmdsm < 0 then sidesum = 45 end
		if cmdsm > 0 then sidesum = -45 end
		
		if cmdfm == 0 then sidesum = $*2 end
		if cmdfm < 0 then sidesum = -sidesum;sum = 180 end
		
		local res = sum + sidesum
		--return FixedAngle( res*FRACUNIT )
		p.mmtbt.dirAsAngle = FixedAngle( res*FRACUNIT )
		
		p.mmtbttimestamp = leveltime
	end
end)

--[[
&pbt usage:

p.mmtbt holds button held time
p.mmtbtprev holds the value in the previous frame for that respective button

Want to check if a player is holding jump?
p.mmtbt.jump > 0

Want to check for a tap on the jump button (not held, but the instant it is pressed)?
p.mmtbt.jump == 1

Want to check when jump was just released?
p.mmtbt.jump < p.mmtbtprev.jump
normally btprev is smaller than bt when the button is held, so if it is larger, it must have been released


or use one of the shortcut functions like a baby
]]

local function checks(p, key)
	if not p then error("Bad argument #1 (expected player)", 3) end
	if not p.mmtbt then
		error("Are you running pbtDaemonmmt()?", 3)
	end
	if key == nil then error("Key argument is nil", 3) end
	if not thosbutons[key] then
		error("Button "..key.." not in the list", 3)
	end
end

rawset(_G, "PBTmmt_Up", function(p, key) checks(p,key)
	return not p.mmtbt[key] -- If 0, true
end)
rawset(_G, "PBTmmt_Down", function(p, key) checks(p,key)
	return not not p.mmtbt[key] -- If >0, true
end)

rawset(_G, "PBTmmt_JustPressed", function(p, key) checks(p,key)
	-- Button has just been pressed after not being pressed
	return p.mmtbt[key] and p.mmtbtprev[key] == 0
end)
rawset(_G, "PBTmmt_JustReleased", function(p, key) checks(p,key)
	-- Button is no longer being pressed after being pressed press press press press press press press press press press 
	return p.mmtbt[key] < p.mmtbtprev[key]
end)